from typing import List, Optional, Any

from cyanprintsdk.api.base_model import CyanBaseModel


class CyanGlobReq(CyanBaseModel):
    root: Optional[str] = None
    glob: str
    exclude: List[str] = []
    type: str


class CyanPluginReq(CyanBaseModel):
    name: str
    config: Any


class CyanProcessorReq(CyanBaseModel):
    name: str
    config: Any
    files: List[CyanGlobReq]


class CyanReq(CyanBaseModel):
    processors: List[CyanProcessorReq]
    plugins: List[CyanPluginReq]



from cyanprintsdk.api.core.core_mapper import CyanMapper
from cyanprintsdk.api.processor.req import ProcessorReq
from cyanprintsdk.api.processor.res import ProcessorRes
from cyanprintsdk.domain.processor.input import ProcessorInput
from cyanprintsdk.domain.processor.output import ProcessorOutput


class ProcessorMapper:
    @staticmethod
    def to_domain(req: ProcessorReq) -> ProcessorInput:
        globs = [CyanMapper.glob_req_to_domain(x) for x in req.globs]
        return ProcessorInput(
            read_directory=req.read_dir,
            write_directory=req.write_dir,
            config=req.config,
            globs=globs,
        )

    @staticmethod
    def to_res(res: ProcessorOutput) -> ProcessorRes:
        return ProcessorRes(output_dir=res.directory)



from typing import Callable, Awaitable

from cyanprintsdk.domain.core.cyan_script import ICyanPlugin
from cyanprintsdk.domain.core.cyan_script_model import CyanPluginInput
from cyanprintsdk.domain.plugin.output import PluginOutput

LambdaPluginFn = Callable[[CyanPluginInput], Awaitable[PluginOutput]]


class LambdaPlugin(ICyanPlugin):
    def __init__(self, f: LambdaPluginFn):
        self._f: LambdaPluginFn = f

    async def plugin(self, i: CyanPluginInput) -> PluginOutput:
        return await self._f(i)



from pydantic import BaseModel, ConfigDict  # type: ignore
from pydantic.alias_generators import to_camel  # type: ignore


class CyanBaseModel(BaseModel):
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
    )



from dataclasses import dataclass
from typing import Any, Dict


@dataclass
class FileOrigin:
    template: str
    layer: int


@dataclass
class ResolvedFile:
    path: str
    content: str
    origin: FileOrigin


@dataclass
class ResolverInput:
    config: Dict[str, Any]
    files: list[ResolvedFile]



from typing import List, Optional, Any

from cyanprintsdk.api.base_model import CyanBaseModel


class CyanGlobRes(CyanBaseModel):
    root: Optional[str]
    glob: str
    exclude: List[str]
    type: str


class CyanPluginRes(CyanBaseModel):
    name: str
    config: Any  # 'Any' is used for the 'unknown' type


class CyanProcessorRes(CyanBaseModel):
    name: str
    config: Any  # 'Any' is used for the 'unknown' type
    files: List[CyanGlobRes]


class CyanRes(CyanBaseModel):
    processors: List[CyanProcessorRes]
    plugins: List[CyanPluginRes]


# No need for explicit export statements in Python



from dataclasses import dataclass
from datetime import date
from typing import Optional, List, Union, Callable


@dataclass
class CheckboxQ:
    message: str
    options: List[str]
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    validate: None = None
    # validate: Optional[Callable[[str], Optional[str]]] = None


@dataclass
class ConfirmQ:
    message: str
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    default: Optional[bool] = None
    error_message: Optional[str] = None
    validate: None = None
    # validate: Optional[Callable[[str], Optional[str]]] = None


@dataclass
class DateQ:
    message: str
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    validate: Optional[Callable[[str], Optional[str]]] = None
    default: Optional[date] = None
    min_date: Optional[date] = None
    max_date: Optional[date] = None


@dataclass
class PasswordQ:
    message: str
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    validate: Optional[Callable[[str], Optional[str]]] = None
    confirmation: Optional[bool] = None


@dataclass
class SelectQ:
    message: str
    options: List[str]
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    validate: None = None
    # validate: Optional[Callable[[str], Optional[str]]] = None


@dataclass
class TextQ:
    message: str
    id: str  # Unique identifier for the question
    desc: Optional[str] = None
    validate: Optional[Callable[[str], Optional[str]]] = None
    default: Optional[str] = None
    initial: Optional[str] = None


# Type hint for a generic question
Question = Union[TextQ, SelectQ, PasswordQ, DateQ, ConfirmQ, CheckboxQ]



from cyanprintsdk.domain.core.question import Question


class OutOfAnswerException(Exception):
    def __init__(self, message: str, question: Question):
        super().__init__(message)
        self.question = question



from pathlib import Path


class VirtualFileReference:
    def __init__(self, base_read: str, base_write: str, relative: str):
        self.base_read = base_read
        self.base_write = base_write
        self.relative = relative

    @property
    def read(self) -> str:
        return str(Path(self.base_read) / self.relative)

    @property
    def write(self) -> str:
        return str(Path(self.base_write) / self.relative)

    def read_file(self):
        with open(self.read, "r", encoding="utf-8") as file:
            content = file.read()
        return VirtualFile(self.base_read, self.base_write, self.relative, content)


class VirtualFile:
    def __init__(self, base_read: str, base_write: str, relative: str, content: str):
        self.base_read = base_read
        self.base_write = base_write
        self.relative = relative
        self.content = content

    @property
    def read(self) -> str:
        return str(Path(self.base_read) / self.relative)

    @property
    def write(self) -> str:
        return str(Path(self.base_write) / self.relative)

    def write_file(self):
        parent_directory = Path(self.write).parent
        parent_directory.mkdir(parents=True, exist_ok=True)
        with open(self.write, "w", encoding="utf-8") as file:
            file.write(self.content)



from cyanprintsdk.api.resolver.req import ResolverReq, ResolvedFileReq, FileOriginReq
from cyanprintsdk.api.resolver.res import ResolverRes
from cyanprintsdk.domain.resolver.input import ResolverInput, ResolvedFile, FileOrigin
from cyanprintsdk.domain.resolver.output import ResolverOutput


class ResolverMapper:
    @staticmethod
    def _to_file_origin(req: FileOriginReq) -> FileOrigin:
        return FileOrigin(
            template=req.template,
            layer=req.layer,
        )

    @staticmethod
    def _to_resolved_file(req: ResolvedFileReq) -> ResolvedFile:
        return ResolvedFile(
            path=req.path,
            content=req.content,
            origin=ResolverMapper._to_file_origin(req.origin),
        )

    @staticmethod
    def to_domain(req: ResolverReq) -> ResolverInput:
        files = [ResolverMapper._to_resolved_file(x) for x in req.files]
        return ResolverInput(
            config=req.config,
            files=files,
        )

    @staticmethod
    def to_res(res: ResolverOutput) -> ResolverRes:
        return ResolverRes(path=res.path, content=res.content)



import pprint

from aiohttp import web  # type: ignore
from pydantic import ValidationError  # type: ignore

from cyanprintsdk.api.plugin.fn import LambdaPluginFn, LambdaPlugin
from cyanprintsdk.api.plugin.mapper import PluginMapper
from cyanprintsdk.api.plugin.req import PluginReq
from cyanprintsdk.api.plugin.res import PluginRes
from cyanprintsdk.api.processor.fn import LambdaProcessorFn, LambdaProcessor
from cyanprintsdk.api.processor.mapper import ProcessorMapper
from cyanprintsdk.api.processor.req import ProcessorReq
from cyanprintsdk.api.processor.res import ProcessorRes
from cyanprintsdk.api.resolver.fn import LambdaResolverFn, LambdaResolver
from cyanprintsdk.api.resolver.mapper import ResolverMapper
from cyanprintsdk.api.resolver.req import ResolverReq
from cyanprintsdk.api.resolver.res import ResolverRes
from cyanprintsdk.api.template.fn import LambdaTemplate, LambdaTemplateFn
from cyanprintsdk.api.template.mapper import TemplateInputMapper, TemplateOutputMapper
from cyanprintsdk.api.template.req import TemplateAnswerReq, TemplateValidateReq
from cyanprintsdk.api.template.res import TemplateRes, TemplateValidRes
from cyanprintsdk.domain.core.cyan_script import (
    ICyanPlugin,
    ICyanProcessor,
    ICyanResolver,
    ICyanTemplate,
)

from cyanprintsdk.domain.plugin.input import PluginInput
from cyanprintsdk.domain.plugin.output import PluginOutput
from cyanprintsdk.domain.plugin.service import PluginService
from cyanprintsdk.domain.processor.input import ProcessorInput
from cyanprintsdk.domain.processor.output import ProcessorOutput
from cyanprintsdk.domain.processor.service import ProcessorService
from cyanprintsdk.domain.resolver.input import ResolverInput
from cyanprintsdk.domain.resolver.output import ResolverOutput
from cyanprintsdk.domain.resolver.service import ResolverService
from cyanprintsdk.domain.template.input import TemplateInput, TemplateValidateInput
from cyanprintsdk.domain.template.output import TemplateOutput
from cyanprintsdk.domain.template.service import TemplateService


async def health_check(request):
    return web.json_response(
        {
            "Message": "OK",
            "Status": "OK",
        }
    )


def start_plugin_with_fn(f: LambdaPluginFn):
    start_plugin(LambdaPlugin(f))


def start_plugin(plugin: ICyanPlugin):
    app = web.Application()

    plugin_service = PluginService(plugin)

    async def plug(request):
        try:
            json = await request.json()
            req = PluginReq(**json)
            pprint.pprint(req)
        except ValidationError as e:
            print(e)
            return web.json_response({"error": str(e)}, status=400)

        # translate to domain
        i: PluginInput = PluginMapper.to_domain(req)
        o: PluginOutput = await plugin_service.plug(i)
        res: PluginRes = PluginMapper.to_res(o)

        return web.json_response(res.model_dump(by_alias=True))

    app.add_routes(
        [
            web.get("/", health_check),
            web.post("/api/plug", plug),
        ]
    )

    web.run_app(app, port=5552)


def start_processor_with_fn(f: LambdaProcessorFn):
    start_processor(LambdaProcessor(f))


def start_processor(processor: ICyanProcessor):
    app = web.Application()

    proc_service = ProcessorService(processor)

    async def process(request):
        try:
            json = await request.json()
            req = ProcessorReq(**json)
            pprint.pprint(req)
        except ValidationError as e:
            print(e)
            return web.json_response({"error": str(e)}, status=400)

        # translate to domain
        i: ProcessorInput = ProcessorMapper.to_domain(req)
        o: ProcessorOutput = await proc_service.process(i)
        res: ProcessorRes = ProcessorMapper.to_res(o)

        return web.json_response(res.model_dump(by_alias=True))

    app.add_routes(
        [
            web.get("/", health_check),
            web.post("/api/process", process),
        ]
    )

    web.run_app(app, port=5551)


def start_template_with_fn(f: LambdaTemplateFn):
    start_template(LambdaTemplate(f))


def start_template(template: ICyanTemplate):
    app = web.Application()

    template_service = TemplateService(template)

    async def template_answer(request):
        try:
            json = await request.json()
            print(json)
            req = TemplateAnswerReq(**json)
            pprint.pprint(req)
        except ValidationError as e:
            print(e)
            return web.json_response({"error": str(e)}, status=400)

        # translate to domain
        i: TemplateInput = TemplateInputMapper.answer_to_domain(req)
        o: TemplateOutput = await template_service.template(i)
        res: TemplateRes = TemplateOutputMapper.to_resp(o)

        res_model = res.model_dump(by_alias=True)

        return web.json_response(res_model)

    async def template_validate(request):
        try:
            json = await request.json()
            req = TemplateValidateReq(**json)
            pprint.pprint(req)
        except ValidationError as e:
            print(e)
            return web.json_response({"error": str(e)}, status=400)

        # translate to domain
        i: TemplateValidateInput = TemplateInputMapper.validate_to_domain(req)
        o: str | None = await template_service.validate(i)
        if o == "":
            o = None
        res: TemplateValidRes = TemplateValidRes(valid=o)

        res_model = res.model_dump(by_alias=True)

        return web.json_response(res_model)

    app.add_routes(
        [
            web.get("/", health_check),
            web.post("/api/template/init", template_answer),
            web.post("/api/template/validate", template_validate),
        ]
    )

    web.run_app(app, port=5550)


def start_resolver_with_fn(f: LambdaResolverFn):
    start_resolver(LambdaResolver(f))


def start_resolver(resolver: ICyanResolver):
    app = web.Application()

    resolver_service = ResolverService(resolver)

    async def resolve(request):
        try:
            json = await request.json()
            req = ResolverReq(**json)
        except ValidationError as e:
            print(e)
            return web.json_response({"error": str(e)}, status=400)

        # translate to domain
        i: ResolverInput = ResolverMapper.to_domain(req)
        o: ResolverOutput = await resolver_service.resolve(i)
        res: ResolverRes = ResolverMapper.to_res(o)

        return web.json_response(res.model_dump(by_alias=True))

    app.add_routes(
        [
            web.get("/", health_check),
            web.post("/api/resolve", resolve),
        ]
    )

    web.run_app(app, port=5553)



from typing import Awaitable, Callable

from cyanprintsdk.domain.core.cyan import Cyan
from cyanprintsdk.domain.core.cyan_script import ICyanTemplate
from cyanprintsdk.domain.core.deterministic import IDeterminism
from cyanprintsdk.domain.core.inquirer import IInquirer

LambdaTemplateFn = Callable[[IInquirer, IDeterminism], Awaitable[Cyan]]


class LambdaTemplate(ICyanTemplate):
    def __init__(self, f: LambdaTemplateFn):
        self._f: LambdaTemplateFn = f

    async def template(self, inquirer: IInquirer, determinism: IDeterminism) -> Cyan:
        return await self._f(inquirer, determinism)



from typing import Optional

from cyanprintsdk.domain.core.cyan_script import ICyanTemplate
from cyanprintsdk.domain.core.question import Question
from cyanprintsdk.domain.service.out_of_answer_error import OutOfAnswerException
from cyanprintsdk.domain.service.stateless_determinism import StatelessDeterminism
from cyanprintsdk.domain.service.stateless_inquirer import StatelessInquirer
from cyanprintsdk.domain.template.input import TemplateInput, TemplateValidateInput
from cyanprintsdk.domain.template.output import (
    TemplateQnAOutput,
    TemplateOutput,
    TemplateFinalOutput,
)


class TemplateService:
    def __init__(self, template: ICyanTemplate):
        self._template = template

    async def template(self, answer: TemplateInput) -> TemplateOutput:
        i = StatelessInquirer(answer.answers)
        d = StatelessDeterminism(answer.deterministic_state)

        try:
            r = await self._template.template(i, d)
            return TemplateFinalOutput(data=r)
        except OutOfAnswerException as e:
            q: Question = e.question
            return TemplateQnAOutput(deterministic_state=d.states, question=q)
        except Exception as e:
            raise e

    async def validate(self, answer: TemplateValidateInput) -> Optional[str]:
        i = StatelessInquirer(answer.answers)
        d = StatelessDeterminism(answer.deterministic_state)

        try:
            await self._template.template(i, d)
            raise RuntimeError("Not supposed to reach here for validation!")
        except OutOfAnswerException as e:
            q: Question = e.question
            validate_result = q.validate(answer.validate) if q.validate else None
            return validate_result
        except Exception as e:
            raise e



from cyanprintsdk.domain.core.cyan_script import ICyanResolver
from cyanprintsdk.domain.resolver.input import ResolverInput
from cyanprintsdk.domain.resolver.output import ResolverOutput


class ResolverService:
    def __init__(self, resolver: ICyanResolver):
        self._resolver = resolver

    async def resolve(self, i: ResolverInput) -> ResolverOutput:
        return await self._resolver.resolve(i)



from typing import Dict, Callable

from cyanprintsdk.domain.core.deterministic import IDeterminism


class StatelessDeterminism(IDeterminism):
    def __init__(self, states: Dict[str, str]):
        self.states = states

    def get(self, key: str, origin: Callable[[], str]) -> str:
        if self.states is None:
            raise RuntimeError("States dictionary is null")

        state = self.states.get(key)

        if state is not None:
            return state

        val = origin()
        self.states[key] = val
        return val



from abc import ABC, abstractmethod
from typing import List, Optional

from cyanprintsdk.domain.core.question import (
    CheckboxQ,
    ConfirmQ,
    PasswordQ,
    SelectQ,
    TextQ,
    DateQ,
)


class IInquirer(ABC):

    @abstractmethod
    async def checkbox(
        self,
        message: str,
        id: str,
        options: List[str],
        desc: Optional[str] = None,
    ) -> List[str]:
        pass

    @abstractmethod
    async def checkboxQ(self, q: CheckboxQ) -> List[str]:
        pass

    @abstractmethod
    async def confirm(self, message: str, id: str, desc: Optional[str] = None) -> bool:
        pass

    @abstractmethod
    async def confirmQ(self, q: ConfirmQ) -> bool:
        pass

    @abstractmethod
    async def password(self, message: str, id: str, desc: Optional[str] = None) -> str:
        pass

    @abstractmethod
    async def passwordQ(self, q: PasswordQ) -> str:
        pass

    @abstractmethod
    async def select(
        self,
        message: str,
        id: str,
        options: List[str],
        desc: Optional[str] = None,
    ) -> str:
        pass

    @abstractmethod
    async def selectQ(self, q: SelectQ) -> str:
        pass

    @abstractmethod
    async def text(self, message: str, id: str, desc: Optional[str] = None) -> str:
        pass

    @abstractmethod
    async def textQ(self, q: TextQ) -> str:
        pass

    @abstractmethod
    async def date_select(
        self, message: str, id: str, desc: Optional[str] = None
    ) -> str:
        pass

    @abstractmethod
    async def date_selectQ(self, q: DateQ) -> str:
        pass



from typing import Dict

from cyanprintsdk.api.base_model import CyanBaseModel

from cyanprintsdk.api.core.answer_req import AnswerReq


class TemplateValidateReq(CyanBaseModel):
    answers: Dict[str, AnswerReq]
    deterministic_states: Dict[str, str]
    validate: str


class TemplateAnswerReq(CyanBaseModel):
    answers: Dict[str, AnswerReq]
    deterministic_states: Dict[str, str]



from typing import Callable, Awaitable

from cyanprintsdk.domain.core.cyan_script import ICyanResolver
from cyanprintsdk.domain.resolver.input import ResolverInput
from cyanprintsdk.domain.resolver.output import ResolverOutput

LambdaResolverFn = Callable[[ResolverInput], Awaitable[ResolverOutput]]


class LambdaResolver(ICyanResolver):
    def __init__(self, f: LambdaResolverFn):
        self._f: LambdaResolverFn = f

    async def resolve(self, i: ResolverInput) -> ResolverOutput:
        return await self._f(i)



from typing import List, Optional, Dict, cast

from cyanprintsdk.domain.core.answer import (
    Answer,
    BoolAnswer,
    StringAnswer,
    StringArrayAnswer,
    is_string_array_answer,
    is_bool_answer,
    is_string_answer,
)
from cyanprintsdk.domain.core.inquirer import IInquirer
from cyanprintsdk.domain.core.question import (
    Question,
    CheckboxQ,
    ConfirmQ,
    PasswordQ,
    SelectQ,
    TextQ,
    DateQ,
)
from cyanprintsdk.domain.service.out_of_answer_error import OutOfAnswerException


class StatelessInquirer(IInquirer):
    def __init__(self, answers: Dict[str, Answer]):
        self._answers = answers

    def _get_answer(self, q: Question) -> Answer:
        if q.id not in self._answers:
            raise OutOfAnswerException("", q)

        return self._answers[q.id]

    async def checkbox(
        self,
        message: str,
        id: str,
        options: List[str],
        desc: Optional[str] = None,
    ) -> List[str]:
        checkbox_q = CheckboxQ(message=message, options=options, desc=desc, id=id)
        return await self.checkboxQ(checkbox_q)

    async def checkboxQ(self, q: CheckboxQ) -> List[str]:
        answer = self._get_answer(q)
        if is_string_array_answer(answer):
            a = cast(StringArrayAnswer, answer)
            return a.answer

        raise TypeError(
            "Incorrect answer type. Expected: StringArrayAnswer. Got: "
            + str(type(answer))
        )

    async def confirm(self, message: str, id: str, desc: Optional[str] = None) -> bool:
        confirm_q = ConfirmQ(message=message, desc=desc, id=id)
        return await self.confirmQ(confirm_q)

    async def confirmQ(self, q: ConfirmQ) -> bool:
        answer = self._get_answer(q)
        if is_bool_answer(answer):
            b = cast(BoolAnswer, answer)
            return b.answer

        raise TypeError(
            "Incorrect answer type. Expected: BoolAnswer. Got: " + str(type(answer))
        )

    async def password(self, message: str, id: str, desc: Optional[str] = None) -> str:
        password_q = PasswordQ(message=message, desc=desc, id=id)
        return await self.passwordQ(password_q)

    async def passwordQ(self, q: PasswordQ) -> str:
        answer = self._get_answer(q)
        if is_string_answer(answer):
            s = cast(StringAnswer, answer)
            return s.answer

        raise TypeError(
            "Incorrect answer type. Expected: StringAnswer. Got: " + str(type(answer))
        )

    async def select(
        self,
        message: str,
        id: str,
        options: List[str],
        desc: Optional[str] = None,
    ) -> str:
        select_q = SelectQ(message=message, options=options, desc=desc, id=id)
        return await self.selectQ(select_q)

    async def selectQ(self, q: SelectQ) -> str:
        answer = self._get_answer(q)
        if is_string_answer(answer):
            s = cast(StringAnswer, answer)
            return s.answer

        raise TypeError(
            "Incorrect answer type. Expected: StringAnswer. Got: " + str(type(answer))
        )

    async def text(self, message: str, id: str, desc: Optional[str] = None) -> str:
        text_q = TextQ(
            message=message,
            desc=desc,
            validate=None,
            id=id,
        )
        return await self.textQ(text_q)

    async def textQ(self, q: TextQ) -> str:
        answer = self._get_answer(q)
        if is_string_answer(answer):
            s = cast(StringAnswer, answer)
            return s.answer

        raise TypeError(
            "Incorrect answer type. Expected: StringAnswer. Got: " + str(type(answer))
        )

    async def date_select(
        self, message: str, id: str, desc: Optional[str] = None
    ) -> str:
        date_q = DateQ(message=message, desc=desc, id=id)
        return await self.date_selectQ(date_q)

    async def date_selectQ(self, q: DateQ) -> str:
        answer = self._get_answer(q)
        if is_string_answer(answer):
            s = cast(StringAnswer, answer)
            return s.answer

        raise TypeError(
            "Incorrect answer type. Expected: StringAnswer. Got: " + str(type(answer))
        )



from typing import cast
from cyanprintsdk.api.core.answer_req import (
    AnswerReq,
    BoolAnswerReq,
    StringAnswerReq,
    StringArrayAnswerReq,
    is_bool_answer_req,
    is_string_answer_req,
    is_string_array_answer_req,
)
from cyanprintsdk.api.core.answer_res import (
    AnswerRes,
    BoolAnswerRes,
    StringAnswerRes,
    StringArrayAnswerRes,
)
from cyanprintsdk.api.core.cyan_req import (
    CyanPluginReq,
    CyanGlobReq,
    CyanProcessorReq,
    CyanReq,
)
from cyanprintsdk.api.core.cyan_res import (
    CyanRes,
    CyanProcessorRes,
    CyanPluginRes,
    CyanGlobRes,
)
from cyanprintsdk.api.core.question_res import (
    QuestionRes,
    ConfirmQuestionRes,
    CheckboxQuestionRes,
    SelectQuestionRes,
    DateQuestionRes,
    PasswordQuestionRes,
    TextQuestionRes,
)
from cyanprintsdk.domain.core.answer import (
    Answer,
    is_string_array_answer,
    is_bool_answer,
    is_string_answer,
    BoolAnswer,
    StringAnswer,
    StringArrayAnswer,
)
from cyanprintsdk.domain.core.cyan import (
    CyanPlugin,
    CyanGlob,
    GlobType,
    CyanProcessor,
    Cyan,
)
from cyanprintsdk.domain.core.question import (
    Question,
    ConfirmQ,
    CheckboxQ,
    SelectQ,
    DateQ,
    PasswordQ,
    TextQ,
)


class CyanMapper:
    @staticmethod
    def plugin_req_to_domain(req: CyanPluginReq) -> CyanPlugin:
        return CyanPlugin(name=req.name, config=req.config)

    @staticmethod
    def glob_req_to_domain(req: CyanGlobReq) -> CyanGlob:
        if req.type == "template":
            glob_type = GlobType.Template
        elif req.type == "copy":
            glob_type = GlobType.Copy
        else:
            raise ValueError(f"Invalid req.type: {req.type}")

        return CyanGlob(
            root=req.root, glob=req.glob, exclude=req.exclude, type=glob_type
        )

    @staticmethod
    def processor_req_to_domain(req: CyanProcessorReq) -> CyanProcessor:
        files = [CyanMapper.glob_req_to_domain(x) for x in req.files]
        return CyanProcessor(name=req.name, config=req.config, files=files)

    @staticmethod
    def cyan_req_to_domain(req: CyanReq) -> Cyan:
        processors = [CyanMapper.processor_req_to_domain(x) for x in req.processors]
        plugins = [CyanMapper.plugin_req_to_domain(x) for x in req.plugins]
        return Cyan(processors=processors, plugins=plugins)

    @staticmethod
    def cyan_to_resp(data: Cyan) -> CyanRes:
        processors = [CyanMapper.processor_to_resp(x) for x in data.processors]
        plugins = [CyanMapper.plugin_to_resp(x) for x in data.plugins]
        return CyanRes(processors=processors, plugins=plugins)

    @staticmethod
    def processor_to_resp(data: CyanProcessor) -> CyanProcessorRes:
        files = [CyanMapper.glob_to_resp(x) for x in data.files]
        return CyanProcessorRes(name=data.name, config=data.config, files=files)

    @staticmethod
    def plugin_to_resp(data: CyanPlugin) -> CyanPluginRes:
        return CyanPluginRes(name=data.name, config=data.config)

    @staticmethod
    def glob_type_to_resp(t: GlobType) -> str:
        if t == GlobType.Template:
            return "template"
        elif t == GlobType.Copy:
            return "copy"
        else:
            raise ValueError(f"Invalid GlobType: {t}")

    @staticmethod
    def glob_to_resp(data: CyanGlob) -> CyanGlobRes:
        return CyanGlobRes(
            root=data.root,
            glob=data.glob,
            exclude=data.exclude,
            type=CyanMapper.glob_type_to_resp(data.type),
        )


class QuestionMapper:
    @staticmethod
    def question_to_resp(q: Question) -> QuestionRes:
        if isinstance(q, SelectQ):
            return QuestionMapper.select_to_resp(q)
        elif isinstance(q, TextQ):
            return QuestionMapper.text_to_resp(q)
        elif isinstance(q, PasswordQ):
            return QuestionMapper.password_to_resp(q)
        elif isinstance(q, DateQ):
            return QuestionMapper.date_to_resp(q)
        elif isinstance(q, ConfirmQ):
            return QuestionMapper.confirm_to_resp(q)
        elif isinstance(q, CheckboxQ):
            return QuestionMapper.checkbox_to_resp(q)
        else:
            raise ValueError(f"Invalid question type: {q}")

    @staticmethod
    def confirm_to_resp(q: ConfirmQ) -> ConfirmQuestionRes:
        return ConfirmQuestionRes(
            default=q.default,
            message=q.message,
            id=q.id,
            error_message=q.error_message,
            desc=q.desc,
            type="confirm",
        )

    @staticmethod
    def checkbox_to_resp(q: CheckboxQ) -> CheckboxQuestionRes:
        return CheckboxQuestionRes(
            message=q.message, id=q.id, desc=q.desc, options=q.options, type="checkbox"
        )

    @staticmethod
    def select_to_resp(q: SelectQ) -> SelectQuestionRes:
        return SelectQuestionRes(
            message=q.message, id=q.id, desc=q.desc, options=q.options, type="select"
        )

    @staticmethod
    def text_to_resp(q: TextQ) -> TextQuestionRes:
        return TextQuestionRes(
            message=q.message,
            id=q.id,
            desc=q.desc,
            default=q.default,
            initial=q.initial,
            type="text",
        )

    @staticmethod
    def password_to_resp(q: PasswordQ) -> PasswordQuestionRes:
        return PasswordQuestionRes(
            message=q.message,
            id=q.id,
            desc=q.desc,
            confirmation=q.confirmation,
            type="password",
        )

    @staticmethod
    def date_to_resp(q: DateQ) -> DateQuestionRes:
        return DateQuestionRes(
            message=q.message,
            id=q.id,
            desc=q.desc,
            default=q.default.isoformat() if q.default else None,
            maxDate=q.max_date.isoformat() if q.max_date else None,
            minDate=q.min_date.isoformat() if q.min_date else None,
            type="date",
        )


class AnswerMapper:
    @staticmethod
    def to_domain(req: AnswerReq) -> Answer:
        if is_bool_answer_req(req):
            bool_answer = cast(BoolAnswerReq, req)
            return BoolAnswer(answer=bool_answer.answer)
        elif is_string_answer_req(req):
            string_answer = cast(StringAnswerReq, req)
            return StringAnswer(answer=string_answer.answer)
        elif is_string_array_answer_req(req):
            string_array_answer = cast(StringArrayAnswerReq, req)
            return StringArrayAnswer(answer=string_array_answer.answer)
        else:
            raise ValueError(f"Invalid answer request type: {req}")

    @staticmethod
    def to_resp(answer: Answer) -> AnswerRes:
        if is_bool_answer(answer):
            return BoolAnswerRes(answer=answer.answer)
        elif is_string_answer(answer):
            return StringAnswerRes(answer=answer.answer)
        elif is_string_array_answer(answer):
            return StringArrayAnswerRes(answer=answer.answer)
        else:
            raise ValueError(f"Invalid answer type: {answer}")



from cyanprintsdk.api.core.core_mapper import AnswerMapper, QuestionMapper, CyanMapper
from cyanprintsdk.api.template.req import TemplateAnswerReq, TemplateValidateReq
from cyanprintsdk.api.template.res import TemplateRes, TemplateQnARes, TemplateFinalRes
from cyanprintsdk.domain.template.input import TemplateInput, TemplateValidateInput
from cyanprintsdk.domain.template.output import (
    TemplateOutput,
    is_qna,
    is_final,
    TemplateQnAOutput,
    TemplateFinalOutput,
)
from typing import cast


class TemplateInputMapper:
    @staticmethod
    def answer_to_domain(req: TemplateAnswerReq) -> TemplateInput:
        answers = {}
        for id, answer in req.answers.items():
            answers[id] = AnswerMapper.to_domain(answer)

        return TemplateInput(
            answers=answers,
            deterministic_state=req.deterministic_states,
        )

    @staticmethod
    def validate_to_domain(req: TemplateValidateReq) -> TemplateValidateInput:
        answers = {}
        for id, answer in req.answers.items():
            answers[id] = AnswerMapper.to_domain(answer)

        return TemplateValidateInput(
            deterministic_state=req.deterministic_states,
            answers=answers,
            validate=req.validate,
        )


class TemplateOutputMapper:
    @staticmethod
    def to_resp(output: TemplateOutput) -> TemplateRes:
        if is_qna(output):
            qna_output = cast(TemplateQnAOutput, output)
            return TemplateQnARes(
                type="questionnaire",
                deterministic_state=qna_output.deterministic_state,
                question=QuestionMapper.question_to_resp(qna_output.question),
            )
        elif is_final(output):
            final_output = cast(TemplateFinalOutput, output)
            return TemplateFinalRes(
                cyan=CyanMapper.cyan_to_resp(final_output.data), type="final"
            )
        else:
            raise ValueError(f"Invalid output type {output}")



from dataclasses import dataclass
from typing import Dict

from cyanprintsdk.domain.core.answer import Answer


@dataclass
class TemplateInput:
    answers: Dict[str, Answer]
    deterministic_state: Dict[str, str]


@dataclass
class TemplateValidateInput:
    answers: Dict[str, Answer]
    deterministic_state: Dict[str, str]
    validate: str



from typing import Any, Dict, List

from cyanprintsdk.api.base_model import CyanBaseModel


class FileOriginReq(CyanBaseModel):
    template: str
    layer: int


class ResolvedFileReq(CyanBaseModel):
    path: str
    content: str
    origin: FileOriginReq


class ResolverReq(CyanBaseModel):
    config: Dict[str, Any]
    files: List[ResolvedFileReq]
